package logic

import (
	"context"
	"encoding/json"
	"fusenapi/constants"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/file"
	"fusenapi/utils/websocket_data"

	"fusenapi/server/render/internal/svc"
	"fusenapi/server/render/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type RenderNotifyLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewRenderNotifyLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RenderNotifyLogic {
	return &RenderNotifyLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *RenderNotifyLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *RenderNotifyLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *RenderNotifyLogic) RenderNotify(req *types.RenderNotifyReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.TaskId == "" {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "invalid param task_id")
	}
	if req.Image == "" {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "invalid param image")
	}
	if req.UserId == 0 && req.GuestId == 0 {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "invalid user_id or guest_id")
	}
	// 上传文件
	var upload = file.Upload{
		Ctx:        l.ctx,
		MysqlConn:  l.svcCtx.MysqlConn,
		AwsSession: l.svcCtx.AwsSession,
	}
	uploadRes, err := upload.UploadFileByBase64(&file.UploadBaseReq{
		FileHash:     req.TaskId,
		FileData:     req.Image,
		UploadBucket: 1,
		ApiType:      2,
		UserId:       req.UserId,
		GuestId:      req.GuestId,
	})
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeFileUploadErr, "failed to upload render resource image")
	}
	//发送消息到对应的rabbitmq
	data := websocket_data.RenderImageNotify{
		TaskId: req.TaskId,
		Image:  uploadRes.ResourceUrl,
	}
	d, _ := json.Marshal(data)
	if err = l.svcCtx.RabbitMq.SendMsg(constants.RABBIT_MQ_RENDER_RESULT_DATA, d); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to send data")
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success")
}
